@@ -606,7 +606,7 @@ def make_anchors(fts, strides, grid_cell_offset=0.5):
         _, _, h, w = fts[i].shape
         sx = torch.arange(end=w, device=device, dtype=dtype) + grid_cell_offset
         sy = torch.arange(end=h, device=device, dtype=dtype) + grid_cell_offset
-        sy, sx = torch.meshgrid(sy, sx, indexing='ij') if TORCH_1_10 else torch.meshgrid(sy, sx)
+        sy, sx = torch.meshgrid(sy, sx, indexing='ij')
         anchor_points.append(torch.stack((sx, sy), -1).view(-1, 2))
         stride_tensor.append(torch.full((h * w, 1), stride, dtype=dtype, device=device))
     return torch.cat(anchor_points), torch.cat(stride_tensor)
@@ -660,6 +660,8 @@ class Detect(nn.Module):
     dynamic = False
     export = False
     shape = None
+    anchors = torch.empty(0)
+    strides = torch.empty(0)
 
     def __init__(self, nc=100, ch=()):
         super().__init__()
